import os
from dotenv import load_dotenv

load_dotenv()

sftp_config = {    "hostname": os.environ.get("hostname_sftp"),
"username": os.environ.get("username_sftp"),
"password": os.environ.get("password_sftp"),
"port": int(os.environ.get("port_sftp")),
"remote_directory": os.environ.get("remote_directory_sftp") , 

}  # Replace with your SFTP configurations
connection_config = {    "username":  os.environ.get("username_oracle"),
"password": os.environ.get("password_oracle"),
"host":  os.environ.get("host_oracle"),
"port":  os.environ.get("port_oracle"),
"service_name":  os.environ.get("service_name_oracle")}  # Replace with your Oracle configurations
azure_blob_config = {    "account_name": os.environ.get("account_name_az_blob_storage"),
"account_key": os.environ.get("account_key_az_blob_storage"),
"container_name": os.environ.get("container_name_az_blob_storage"),
"flow_name":os.environ.get("flow_name")}  # Replace with your Azure Blob Storage configurations
flow_config = {
    "table_name" : os.environ.get("table_name"),
    "flow_name" : os.environ.get("flow_name"),
    "primary_key" : os.environ.get("primary_key"),
     "regex_pattern" : os.environ.get("regex_pattern"),
     "date_fields" : os.environ.get('date_fields', '').split(',')
}

azure_table_config = {
    "account_key": os.environ.get("account_key_az_blob_storage"),
    "account_name": os.environ.get("account_name_az_blob_storage"),
    "flow_name":os.environ.get("flow_name"),
   
    
}

sql_server_config = {
    "sql_server" : os.environ.get("sql_server"),
    "username" :  os.environ.get("username_sqlserver") ,
    "database" : os.environ.get("database_sqlserver") ,
    "password" : os.environ.get("password_server")
}




import pysftp
import logging

class SFTPClient:
    def __init__(self, hostname, port, username, password):
        self.hostname = hostname
        self.port = port
        self.username = username
        self.password = password
        self.sftp = None
    
    def connect(self):
        try:
            logging.info('Trying to connect to sftp')
            self.sftp = pysftp.Connection(self.hostname, port=self.port, username=self.username, password=self.password)
            logging.info("Connected to SFTP server")
        except Exception as e:
            logging.error(f"Error connecting to SFTP server: {e}")
    
    def upload_file(self, local_path, remote_path):
        try:
            logging.info(f'Uploading file from {local_path} to sftp {remote_path}')
            self.sftp.put(local_path, remote_path)
            logging.info(f"File uploaded successfully to {remote_path}")
        except Exception as e:
            logging.info(f"Error uploading file: {e}")
    
    def download_file(self, remote_path, local_path):
        try:
            logging.info(f'Downloading file from {remote_path} to sftp {local_path}')
            self.sftp.get(remote_path, local_path)
            logging.info(f"File downloaded successfully to {local_path}")
        except Exception as e:
            logging.info(f"Error downloading file: {e}")
    
    def close(self):
        if self.sftp:
            self.sftp.close()
            logging.info("Connection closed")




from setuptools import setup, find_packages
import os

# Read the version from the environment variable
version = None  # Initialize version as None

if "VERSION" in os.environ:
  version = os.environ["VERSION"]
else:
  version = "1.0.0"  # Replace 1.0.0 with your default version

setup(
    name='arion_library',  # Replace with your package name
    version=version,  # Replace with your package version
    author='Heni Nechi',  # Replace with your name
    author_email='h.nechi@arion-tech.com',  # Replace with your email
    url='https://github.com/Ariontech/ArionLibrary.git',  # Replace with your repository URL
    packages=find_packages(),  # Automatically find all packages
    python_requires='>=3.8',  # Specify Python version requirements
    install_requires=[
        'pyodbc',
        'pytest',
        'azure-core'
        'azure-data-tables'
        'azure-storage-blob'
        'python-dotenv'
        'pytest'
    ],
)



import pytest
from unittest.mock import Mock, patch
from azure.storage.blob import BlobServiceClient, BlobClient, ContainerClient
from ..lib.azureBlobStorage import AzureBlobProcessor

@pytest.fixture
def mock_blob_service_client():
    return Mock(spec=BlobServiceClient)

@pytest.fixture
def mock_container_client():
    return Mock(spec=ContainerClient)

@pytest.fixture
def azure_blob_processor(mock_blob_service_client, mock_container_client):
    with patch('azure.storage.blob.BlobServiceClient.from_connection_string', return_value=mock_blob_service_client):
        with patch.object(mock_blob_service_client, 'get_container_client', return_value=mock_container_client):
            yield AzureBlobProcessor("test_account_name", "test_account_key", "test_container", "test_flow_name")

def test_azure_blob_processor_init(azure_blob_processor, mock_blob_service_client, mock_container_client):
    assert azure_blob_processor.blob_service_client == mock_blob_service_client
    assert azure_blob_processor.container_client == mock_container_client
    assert azure_blob_processor.azure_blob_config == {"account_name": "test_account_name", "account_key": "test_account_key", "container" :"test_container", "flow_name" : "test_flow_name"}

def test_read_blob_files(azure_blob_processor, mock_container_client):
    mock_blob = Mock()
    mock_blob.name = "test_blob_name.txt"
    mock_blob_client = Mock()
    mock_blob_client.download_blob.return_value.readall.return_value = b"Test Blob Content"
    mock_container_client.list_blobs.return_value = [mock_blob]

    with patch.object(mock_container_client, 'get_blob_client', return_value=mock_blob_client):
        files = list(azure_blob_processor.read_blob_files())

    assert len(files) == 1
    assert files[0]['file_name'] == "test_blob_name.txt"
    assert files[0]['file_content'].read() == b"Test Blob Content"

def test_push_files_to_blob(azure_blob_processor, mock_container_client):
    mock_file_info = ("test_file.txt", b"Test File Content")
    azure_blob_processor.push_files_to_blob([mock_file_info])

    mock_container_client.upload_blob.assert_called_once_with(name="test_flow_name/21-03-2024/test_file.txt", data=mock_file_info[1], overwrite=True)



from azure.data.tables import TableServiceClient
from azure.core.exceptions import ResourceExistsError
import logging
from datetime import datetime

class AzureDataTablesClient:
    def __init__(self, connection_string, table_name):
        self.connection_string = connection_string
        self.table_name = table_name
        self.table_service_client = None

    def connect(self):
        try:
            logging.info('Trying to connect to Azure Data Tables')
            self.table_service_client = TableServiceClient.from_connection_string(self.connection_string)
            logging.info("Connected to Azure Data Tables")
        except Exception as e:
            logging.error(f"Error connecting to Azure Data Tables: {e}")

    def insert_batch_entities(self, entities,PartitionKey, RowKey,columnstoinsert, batch_size=1):
        try : 
            logging.info(f'Preparing data to insert into {self.table_name} table')
            table_client = self.table_service_client.get_table_client(self.table_name)
            entities_to_insert = []
            for _,row in entities.iterrows():
                entity = {'PartitionKey': PartitionKey, 'RowKey': row[RowKey]}
                for col in columnstoinsert:
                    entity[col] = row[col]
                entities_to_insert.append(("upsert", entity))
                if len(entities_to_insert) == batch_size : 
                     logging.info(f'batch to insert : {entities_to_insert}')
                     table_client.submit_transaction(entities_to_insert)
                     entities_to_insert = []
            if len(entities_to_insert) > 0 : 
                logging.info(f'batch to insert : {entities_to_insert}')
                table_client.submit_transaction(entities_to_insert)
                
            logging.info(f'All lines are successfully inserted into {self.table_name} table.')

        except ValueError as e : 
             logging.error(f'An error was occured while trying to insert data into {self.table_name} table : {e}')

    def query_entities(self,filter_condition,batch_size=1):
        try:

            table_client = self.table_service_client.get_table_client(self.table_name)
            entities = []
            logging.info(f'Executing query : {filter_condition}')
            for entity_page in table_client.query_entities(query_filter=filter_condition, results_per_page=batch_size).by_page():
                entities.extend(list(entity_page))
                break
            logging.info(f'Query Results : {entities}')
            return entities
        except ValueError as e:
            logging.error(f'An error occured while trying to query data from {self.table_name} table : {e}')
            


